package me.unurled.sacredrealms.sr.components.player;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.destroystokyo.paper.event.player.PlayerArmorChangeEvent;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import java.util.HashMap;
import java.util.UUID;
import me.unurled.sacredrealms.sr.data.DataHandler;
import me.unurled.sacredrealms.sr.data.DataManager;
import me.unurled.sacredrealms.sr.data.gson.InventoryDeserializer;
import me.unurled.sacredrealms.sr.data.gson.InventorySerializer;
import me.unurled.sacredrealms.sr.data.gson.PotionEffectDeserializer;
import me.unurled.sacredrealms.sr.data.gson.PotionEffectSerializer;
import me.unurled.sacredrealms.sr.managers.Manager;
import me.unurled.sacredrealms.sr.utils.Items;
import org.bukkit.event.EventHandler;
import org.bukkit.event.player.PlayerItemHeldEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.potion.PotionEffect;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class PlayerManager extends Manager {

  public static final String PLAYER_KEY = "sr.players.";
  private final HashMap<UUID, SRPlayer> players;

  public PlayerManager() {
    super();
    players = new HashMap<>();
  }

  /** Save the data */
  @Override
  public void saveData() {
    DataManager dm = DataManager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't save player data.");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    // save all players
    Gson gson =
        new GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .registerTypeAdapter(Inventory.class, new InventorySerializer())
            .registerTypeAdapter(PotionEffect.class, new PotionEffectSerializer())
            .create();
    for (SRPlayer player : players.values()) {
      dh.set(PLAYER_KEY + player.getUuid(), gson.toJson(player));
    }

    players.clear();
  }

  @Nullable
  public SRPlayer getPlayer(UUID uuid) {
    return players.get(uuid);
  }

  public void addPlayer(@NotNull SRPlayer player) {
    players.put(player.getUuid(), player);
  }

  public void removePlayer(UUID uuid) {
    players.remove(uuid);
  }

  public boolean isSRPlayer(UUID uuid) {
    return players.containsKey(uuid);
  }

  @EventHandler
  public void onPlayerJoin(PlayerJoinEvent e) {
    DataManager dm = DataManager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't load player " + e.getPlayer().getName() + "'s data" + ".");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    Gson gson =
        new GsonBuilder()
            .registerTypeAdapter(Inventory.class, new InventoryDeserializer())
            .registerTypeAdapter(PotionEffect.class, new PotionEffectDeserializer())
            .excludeFieldsWithoutExposeAnnotation()
            .create();
    String json = dh.get(PLAYER_KEY + e.getPlayer().getUniqueId());
    SRPlayer srPlayer = gson.fromJson(json, SRPlayer.class);
    if (srPlayer == null) {
      srPlayer = new SRPlayer(e.getPlayer());
    }

    addPlayer(srPlayer);

    e.getPlayer().getInventory().clear();
    e.getPlayer().getInventory().setContents(srPlayer.getInventory().getContents());

    e.getPlayer().updateInventory();
  }

  @EventHandler
  public void onPlayerQuit(@NotNull PlayerQuitEvent e) {
    SRPlayer player = getPlayer(e.getPlayer().getUniqueId());
    if (player == null) {
      return;
    }
    // save for player
    DataManager dm = DataManager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't save player " + e.getPlayer().getName() + "'s data" + ".");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    Gson gson =
        new GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .registerTypeAdapter(Inventory.class, new InventorySerializer())
            .registerTypeAdapter(PotionEffect.class, new PotionEffectSerializer())
            .create();
    dh.set("sr.players." + e.getPlayer().getUniqueId(), gson.toJson(player));

    removePlayer(e.getPlayer().getUniqueId());
  }

  @EventHandler
  public void onPlayerInventorySlotChangeEvent(@NotNull PlayerItemHeldEvent e) {
    SRPlayer player = getPlayer(e.getPlayer().getUniqueId());
    if (player == null) {
      return;
    }
    player.setInventory(e.getPlayer().getInventory());

    ItemStack oldItem = e.getPlayer().getInventory().getItem(e.getPreviousSlot());
    ItemStack newItem = e.getPlayer().getInventory().getItem(e.getNewSlot());

    Items.updatePlayer(e.getPlayer(), oldItem, newItem);
  }

  @EventHandler
  public void onPlayerArmorChange(PlayerArmorChangeEvent e) {
    SRPlayer player = getPlayer(e.getPlayer().getUniqueId());
    if (player == null) {
      return;
    }
    player.setInventory(e.getPlayer().getInventory());

    Items.updatePlayer(e.getPlayer(), e.getOldItem(), e.getNewItem());
  }
}
